/*
 # ---------------------------------------------------------------------
 #
 # Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image
 #                        pour la SantÈ)
 # Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
 # Previous Authors : Laurent Guigues, Jean-Pierre Roux
 # CreaTools website : www.creatis.insa-lyon.fr/site/fr/creatools_accueil
 #
 #  This software is governed by the CeCILL-B license under French law and
 #  abiding by the rules of distribution of free software. You can  use,
 #  modify and/ or redistribute the software under the terms of the CeCILL-B
 #  license as circulated by CEA, CNRS and INRIA at the following URL
 #  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
 #  or in the file LICENSE.txt.
 #
 #  As a counterpart to the access to the source code and  rights to copy,
 #  modify and redistribute granted by the license, users are provided only
 #  with a limited warranty  and the software's author,  the holder of the
 #  economic rights,  and the successive licensors  have only  limited
 #  liability.
 #
 #  The fact that you are presently reading this means that you have had
 #  knowledge of the CeCILL-B license and that you accept its terms.
 # ------------------------------------------------------------------------ */


/*=========================================================================
  Program:   bbtk
  Module:    $RCSfile: bbvtkPolyDataToActor.cxx,v $
  Language:  C++
  Date:      $Date: 2009/05/28 08:55:44 $
  Version:   $Revision: 1.15 $
=========================================================================*/

/**
 *  \file 
 *  \brief 
 */


#ifdef _USE_VTK_

#include "vtkProperty.h"
#include "vtkLinearTransform.h"
#include "vtkCleanPolyData.h"
#include "vtkFieldData.h"

#include "bbvtkPolyDataToActor.h"
#include "bbvtkPackage.h"

#include "vtkRenderWindow.h"

namespace bbvtk
{
   BBTK_ADD_BLACK_BOX_TO_PACKAGE(vtk,PolyDataToActor)
   BBTK_BLACK_BOX_IMPLEMENTATION(PolyDataToActor,bbtk::AtomicBlackBox);

   void PolyDataToActor::bbUserSetDefaultValues() 
   { 
	   actorAdded=false;
	   std::vector<double> colour;
	   colour.push_back(1.0);
	   colour.push_back(1.0);
	   colour.push_back(0.5);
	   bbSetInputColour(colour);
	   bbSetInputActive(true);
	   bbSetInputIn(NULL);
	   bbSetInputRenderer(NULL);
	   bbSetInputTransform(NULL);
	   bbSetInputOpacity(1);
	   bbSetInputRepresentation(2);
	   bbSetInputLineWidth(1);
	   bbSetInputScalarVisibility(false);
	   polydatamapper = NULL;
	   vtkactor       = NULL;
	   
	   bbSetInputScalarVisibilityOn_LookupTable(NULL);
	   bbSetInputScalarVisibilityOn_NameColorArray("");
       bbSetInputIn(NULL);
   }


	//---------------------------------------------------------------------
   void PolyDataToActor::bbUserInitializeProcessing() 
   { 	 
     polydatamapper = vtkPolyDataMapper::New();
     vtkactor       = vtkActor::New();
//     polydatamapper->SetInput(marchingcubes->GetOutput());
     vtkactor->SetMapper(polydatamapper);

//EED 2020-04-21 vtk8 deprecated
//     polydatamapper->ImmediateModeRenderingOn();
   }

	//---------------------------------------------------------------------
	void PolyDataToActor::bbUserFinalizeProcessing() 
	{ 
		if (polydatamapper!=NULL)
		{
			polydatamapper->Delete();
			polydatamapper=NULL;
		}
		if (vtkactor!=NULL)
		{
			vtkactor->Delete();
			vtkactor=NULL;
		}		
	}
	
	
	
	
/*

-----------------------------------------------

https://vtk.org/Wiki/VTK/Examples/Cxx/Visualization/ScalarBarActor
https://vtk.org/Wiki/VTK/Examples/Cxx/VisualizationAlgorithms/TubesFromSplines
https://vtk.org/Wiki/VTK/Examples/Cxx/Visualization/ElevationBandsWithGlyphs

vtkSmartPointer<vtkFloatArray> scalars = vtkSmartPointer<vtkFloatArray>::New();
  scalars  set value ....
poly->GetPointData()->SetScalars(scalars);  or   poly->GetPointData()->SetActiveScalars("<NameArray>");

vtkSmartPointer<vtkLookupTable> hueLut =    vtkSmartPoint      |                                                 ^~
er<vtkLookupTable>::New();  
  hueLut Build ....
 
  mapper->SetInputData(poly);
  mapper->ScalarVisibilityOn();
   	mapper->SetScalarModeToUsePointData();   Point of Cell ...
  	  
    default: mapper->SetColorModeToDirectScalars() Busca scalars  3 componentes [R,G,B]  float  0..1  or  integer 0..255
    else:
      mapper->SetColorModeToMapScalars()  El vector scalar solo tiene un componente
      mapper->SetLookupTable( hueLut );  
	  
	  	To change Array
	  	mapper->SelectColorArray("<NameVector>");   
   
	mapper->SetUseLookupTableScalarRange( true/false )    true:Para compartir lookuptable con otros mappers 
		si false then
		mapper->SetScalarRange( min, max);    Range especifico este mapper
	    mapper->SetScalarRange( tubePolyData->GetScalarRange() );

------------------------------------------
   
     mapper->ScalarVisibilityOff();      ->  SetScalarModeToDefault()
        Utiliza el color del actor
  
----------------------------------------

*/	
	
	
//---------------------------------------------------------------------

void PolyDataToActor::DoProcess()
{
    bool ok_removeActor=false;
    if (bbGetInputRenderer()==NULL)
    {
        printf("EED Warnning! PolyDataToActor::DoProcess  missing Renderer.\n");
    }
    if (bbGetInputActive()==true)
    {
        if (bbGetInputIn()!=NULL)
        {
//EED 2017-01-01 Migration VTK7
#if VTK_MAJOR_VERSION <= 5
             polydatamapper->SetInput( bbGetInputIn() );
#else
             polydatamapper->SetInputData( bbGetInputIn() );
#endif
             vtkactor->GetProperty()->SetRepresentation( bbGetInputRepresentation() );
            if (bbGetInputRepresentation()==0)
            {
                 vtkactor->GetProperty()->SetAmbient(1);
                 vtkactor->GetProperty()->SetDiffuse(1);
                 vtkactor->GetProperty()->SetSpecular(0);
printf("EED WARNNING!  PolyDataToActor::DoProcess  which is the default values of Ambient, Diffuse, Specular for points option? \n");
            } else if (bbGetInputRepresentation()==1)
            {
                 vtkactor->GetProperty()->SetAmbient(1);
                 vtkactor->GetProperty()->SetDiffuse(1);
                 vtkactor->GetProperty()->SetSpecular(0);
            } else if (bbGetInputRepresentation()==2)
            {
                 vtkactor->GetProperty()->SetAmbient(0);
                 vtkactor->GetProperty()->SetDiffuse(1);
                 vtkactor->GetProperty()->SetSpecular(0);
            }
             vtkactor->GetProperty()->SetLineWidth( bbGetInputLineWidth() );
             vtkactor->GetProperty()->SetColor( bbGetInputColour()[0],
                            bbGetInputColour()[1],
                            bbGetInputColour()[2] );
             vtkactor->GetProperty()->SetOpacity( bbGetInputOpacity() );
             if ( bbGetInputTransform()!=NULL )
             {
                vtkactor->SetUserTransform( bbGetInputTransform() );
             }
             bbSetOutputOut( vtkactor );
             // Interface Update
             if ((actorAdded==false) && (bbGetInputRenderer()!=NULL ))
             {
               actorAdded=true;
               bbGetInputRenderer()->AddActor( vtkactor );
             }  // actorAdded
             if (bbGetInputScalarVisibility()==true )
             {
                polydatamapper->ScalarVisibilityOn();
                if (bbGetInputScalarVisibilityOn_LookupTable()!=NULL)
                {
//					polydatamapper->SetScalarModeToDefault();
//					polydatamapper->SetScalarModeToUseCellData();
//					polydatamapper->SetScalarModeToUseCellFieldData();
//					polydatamapper->SetScalarModeToUseFieldData();    //    1/2  	hausdorff->SetTargetDistanceMethodToPointToCell();
//					polydatamapper->SetScalarModeToUsePointData();	 // *
                    polydatamapper->SetScalarModeToUsePointFieldData();
                    polydatamapper->SetColorModeToMapScalars();
                    polydatamapper->SetLookupTable( bbGetInputScalarVisibilityOn_LookupTable() );
                    if (bbGetInputScalarVisibilityOn_NameColorArray()!="")
                    {
                        polydatamapper->SelectColorArray( bbGetInputScalarVisibilityOn_NameColorArray().c_str() );
                    }
                    if (bbGetInputScalarVisibilityOn_ScalarRange().size()==2)
                    {
                        polydatamapper->SetScalarRange(  bbGetInputScalarVisibilityOn_ScalarRange()[0] , bbGetInputScalarVisibilityOn_ScalarRange()[1]   );
                    }
                 } // if ScalarVisibilityOn_LookupTable
             } else {
                 polydatamapper->ScalarVisibilityOff();
             } // ScalarVisibility
        } else {
            ok_removeActor=true;
            printf("EED Warnning! PolyDataToActor::DoProcess   In (PolyData) not defined. \n"  );
        }// if In !=NULL
    } else {
         ok_removeActor=true;
    } // Active
    
    if (ok_removeActor==true)
    {
        // Interface Update
        if ((actorAdded==true) && (bbGetInputRenderer()!=NULL ))
        {
          actorAdded=false;
          bbGetInputRenderer()->RemoveActor( vtkactor );
        }  // actorAdded
    } // if ok_removeActor
    
}

} // EO namespace bbtk

#endif //_USE_VTK_

